using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;





namespace IDP
{
    class DigitalFlower : IRender
    {
        private const long DF_POINTS = 1000000;
        private const double DF_MAX_PRECISION = 0.00001;
        private const double DF_AVG_PRECISION = 0.001;
        private const double DF_MIN_PRECISION = 0.1;
        private const double PI = 3.1415192;
        private const double RATIO = 0.99999;
        private const double cM = 1E-30;
        private Point[] m_aPoints = new Point[DF_POINTS];
        private static long lIdx;

        public long DFPointArrLim
        {
            get
            {
                return DF_POINTS;
            }
        }
       
        public  DigitalFlower()
        {
            lIdx = 0;
        }

        public bool DFGenerate(long lWidth, long lHeight, double dblPrecision)
        {
            double z, r, u, v;
            double a = 1, b = 1;
            double sx = lWidth, sy = lHeight;
            double hx = sx / 2, hy = sy / 2;
            double m = cM;
            long lIdx = 0;
            double dblDFRenderPrecison = DF_AVG_PRECISION;
            pvClearArray();

            if (dblPrecision == 0 ||  dblPrecision > 0.01 || dblPrecision < 0.00001 )
            {
                dblDFRenderPrecison = DF_AVG_PRECISION;
            }
            else
            {
                dblDFRenderPrecison = dblPrecision;
            }

            for (z = 0; z <= 2 * PI; z += 0.1)
            {
                r = Math.Abs((2 * Math.Cos(32 * z)) + (2 * Math.Sin(2 * z)));
                if (m < r)
                {
                    m = r + 0.1;
                }
            }
            //0.1 - 0.00001
            for (z = 0; z <= 2 * PI; z += dblDFRenderPrecison)
            {
                // calculate r, u, v
                r = Math.Abs((2 * Math.Cos(32 * z)) + (2 * Math.Sin(2 * z)));
                u = hx + hy * RATIO * Math.Cos(a * z) * r / m;
                v = hy + hy * Math.Sin(b * z) * r / m;
                // Fill points array
                if ((v < 0) || (v > sy))
                {
                    // do nothing
                }
                else
                {
                    m_aPoints[lIdx].X = (int)u;
                    m_aPoints[lIdx].Y = (int)v;
                    lIdx++;
                }
            }
            return true;
        }

        public bool DFRender(System.Drawing.Graphics oGraphics, System.Drawing.Pen objPen)
        {
            long lIdx = 0;
            for (lIdx = 0; lIdx < DF_POINTS - 1; lIdx++)
            {
                if (m_aPoints[lIdx].X != 0 && m_aPoints[lIdx].Y != 0)
                {
                    Rectangle oDot =
                        new Rectangle(m_aPoints[lIdx].X, m_aPoints[lIdx].Y, 1, 1);
                    oGraphics.DrawRectangle(objPen, oDot);
                }
            }
            return true;
        }

        private void pvClearArray()
        {
            long lIdx = 0;
            for (lIdx = 0; lIdx < DF_POINTS; lIdx++)
            {
                m_aPoints[lIdx].X = 0;
                m_aPoints[lIdx].Y = 0;
            }
        }

        public bool DFRenderBitmap(Bitmap oBmp)
        {
            if (lIdx < (DF_POINTS))
            {
                if ((m_aPoints[lIdx].X != 0) && (m_aPoints[lIdx].Y != 0))
                {
                    oBmp.SetPixel(m_aPoints[lIdx].X
                                    , m_aPoints[lIdx].Y
                                    , System.Drawing.Color.White);

                }
                lIdx++;
            }
            return true; 
        }

        public bool IRenderToScreen(System.Drawing.Graphics oGraphics
                          , System.Drawing.Pen oPen)
        {
            long lIdx = 0;
            for (lIdx = 0; lIdx < DF_POINTS - 1; lIdx++)
            {
                if (m_aPoints[lIdx].X != 0 && m_aPoints[lIdx].Y != 0)
                {
                    Rectangle oDot =
                        new Rectangle(m_aPoints[lIdx].X, m_aPoints[lIdx].Y, 1, 1);
                    oGraphics.DrawRectangle(oPen, oDot);
                }
            }
            return true;
        }

        public bool IRenderToScreenDelayed(System.Drawing.Graphics oGraphics
                          , System.Drawing.Pen oPen)
        {
            if (lIdx < (DF_POINTS ))
            {
                Point oPoint = new Point();

                oPoint = m_aPoints[lIdx];

                Rectangle oDot =
                    new Rectangle(oPoint.X, oPoint.Y, 1, 1);
                if ((oPoint.X != 0) && (oPoint.Y != 0))
                {
                    oGraphics.DrawRectangle(oPen, oDot);
                }
                lIdx++;
            }
            return true;
        }

        public bool IRenderToFile(string sFileName)
        {
            return true;
        }

        public bool IRenderToBitmapDelayed(Bitmap oBmp)
        {
            if (lIdx < (DF_POINTS))
            {
                if ((m_aPoints[lIdx].X != 0) && (m_aPoints[lIdx].Y != 0))
                {
                    oBmp.SetPixel(m_aPoints[lIdx].X
                                    , m_aPoints[lIdx].Y
                                    , System.Drawing.Color.White);

                }
                lIdx++;
            }
            return true; 
        }

        public bool IRenderToBitmap(Bitmap oBmp)
        {
            long lIdx = 0;
            for (lIdx = 0; lIdx < DF_POINTS; lIdx++)
            {
                if (m_aPoints[lIdx].X != 0 && m_aPoints[lIdx].Y != 0)
                {
                    oBmp.SetPixel(m_aPoints[lIdx].X, m_aPoints[lIdx].Y, System.Drawing.Color.White);
                }
            }
            return true;
        }

    }// End of class 
}// End of namespace